#ifndef CMDALIAS_H
#define CMDALIAS_H
/** \file Definitions for command string aliases.
*/
/*
   
   Copyright  2002-2009 M.Kraemer, D-64291 Darmstadt, Seegartenstr. 32, 
                         all rights reserved

   $Log: cmdalias.h,v $
   Revision 1.1.1.1  2009/05/08 00:34:35  kraemer


   Revision 1.1.2.2  2009/05/08 00:09:17  kraemer
   64bit-clean, ce return

   Revision 1.1.2.1  2005/03/15 14:57:11  kraemer
   *** empty log message ***


*/

#include "cmdsys.h"
#include "cmderr.h"

#define CMDALIAS_DEBUG CMD_DEBUG
                                      /** \struct SCMDALIASITEM
                                          \brief a single alias entry.
                                      */ 
struct SCMDALIASITEM {
   char *pcAlias;                     /**< alias replacement */
   char *pcStr;                       /**< string to be aliased */
};
                                      /** \struct SCMDALIAS
                                          \brief structure for all alias activity.
                                      */ 
struct SCMDALIAS {
   unsigned int uiFlag;               /**< status flag bits */
#define CMDALIAS_ACTIVE 0x00000001    /**< alias objects are setup and active */                                      
   int nItem, mItem;                  /**< current, max # of items */ 
   struct SCMDALIASITEM *psItem;      /**< alias item list */
   struct SCMDLOCK sCL;               /**< lock handle */
};
                                      /*  */ 
                                      /* item slot increment  */ 
#define CMDALIAS_ITEMINC 8
                                      /* function specific flags */
/*
   _cmdaliasitemFree    ( cmdaliasitem, flag )
                        o release alias item's resources   
                      3 o to test
  
   _cmdaliasDeleteItem  ( cmdalias, stritem, flag )
                        o delete specified entry in alias list 
                      9 x to test
                        o calls
                          o _cmdaliasitemFree()
  
   _cmdaliasFree        ( cmdalias, flag )
                        o release resources 
                      4 x to test
                        o problem: shutdown when other tasks hold the lock
                        o calls:
                          o _cmdaliasitemFree()
                          o _cmdaliasInit()
  
   _cmdaliasGetItem     ( cmdalias, cmdaliasitem, stralias, flag )
                        o locate item in list, copy w/ strdup()
                      8 x to test
                        o calls:
                          o _cmdaliasitemFree()
  
   _cmdaliasImportItem  ( cmdalias, cmdaliasitem, flag )
                        o store an entry in alias list 
                      6 x to test
                        o calls:
                          o _cmdaliasitemFree()
  
   _cmdaliasInit        ( cmdalias, flag )
                        o initialize to "zero" state 
                      2 x to test
  
   _cmdaliasInspect     ( cmdalias, flag )
                        o inspect  
                      1 x to test
  
   _cmdaliasList        ( cmdalias, flag )
                        o list alias entries 
                      7 x to test
  
   _cmdaliasSetup       ( cmdalias, flag )
                        o initialize for use 
                      5 x to test
                        o calls:
                          o _cmdaliasInit()
  
*/
                                      /* function prototypes */
struct SCMDERR _cmdaliasitemFree  ( struct SCMDALIASITEM *, unsigned int );
struct SCMDERR _cmdaliasDeleteItem( struct SCMDALIAS *, const char *, unsigned int );
struct SCMDERR _cmdaliasFree      ( struct SCMDALIAS *, unsigned int );
struct SCMDERR _cmdaliasGetItem   ( struct SCMDALIAS *, struct SCMDALIASITEM *, const char *, unsigned int );
struct SCMDERR _cmdaliasImportItem( struct SCMDALIAS *, struct SCMDALIASITEM *, unsigned int );
struct SCMDERR _cmdaliasInit      ( struct SCMDALIAS *, unsigned int );
struct SCMDERR _cmdaliasInspect   ( struct SCMDALIAS *, unsigned int );
struct SCMDERR _cmdaliasList      ( struct SCMDALIAS *, unsigned int );
struct SCMDERR _cmdaliasSetup     ( struct SCMDALIAS *, unsigned int );

struct SCMDALIAS _cmdaliasTest( unsigned int );

int _cmdaliasError( struct SCMDERR *, unsigned int );

/*
   --- error codes
*/
enum { ERRCMDALIAS_SYS=1,
       ERRCMDALIAS_BADADDR, 
       ERRCMDALIAS_SYNTAX,
       ERRCMDALIAS_NOTFOUND,
       ERRCMDALIAS_BADNAME,
       ERRCMDALIAS_BADCMD,
};

#endif
