#ifndef MMATHFCT_H 
#define MMATHFCT_H

/** \file Definitions for special math functions.
*/
/*
   $Log: mmathfct.h,v $
   Revision 1.5  2015/01/05 19:13:51  kraemer
   *** empty log message ***

   Revision 1.4  2014/02/19 18:07:08  kraemer
   more platform branches

   Revision 1.3  2012/08/16 18:03:14  kraemer
   common rounding definition

   Revision 1.2  2011/09/15 16:47:15  kraemer
   sincos renamed (name clash)

   Revision 1.1.1.1  2009/05/07 12:58:12  kraemer


   Revision 1.1.2.2  2009/05/07 12:35:02  kraemer
   *** empty log message ***

   Revision 1.1.2.1  2009/05/06 22:26:28  kraemer
   *** empty log message ***


*/

#include <sys/types.h>
#include <float.h>
#include <math.h>

#include "osversion.h"
#include "utypes.h"
#include "mtypes.h"

                                      /* */
double tDist   ( double, ssize_t );
double tProb   ( double, ssize_t );
double tQuantil( double, ssize_t );
double chiDist   ( double, ssize_t );
double chiProb   ( double, ssize_t );
double chiQuantil( double, ssize_t );
                                      /* */
 
double beta    ( double, double );
double betainc ( double, double, double );

/*
   --- ROUND
*/
#undef MMATHFCT_HAS_ROUND
                                      /* platforms w/ native round functions */
#if 0 \
    || ( defined(_AIX) && defined(_AIX52) ) \
    || ( defined(__linux__) ) \
    || ( defined(__sun) && defined(__sparc) ) \
    || defined(__APPLE_CC__) \
    || 0
#define ROUND(x) round(x)
#define MMATHFCT_HAS_ROUND
                                      /* platforms w/o native round functions */
#elif 0 \
      || ( defined(_AIX) && ( !defined(_AIX52) ) ) \
      || ( defined(__hpux) && ( __STDC_VERSION__ < 199901L) ) \
      || ( defined(__osf__) ) \
      || 0

#define ROUND(x) ( (x) >= (0.0) ) ? (floor((x)+0.5)) : (ceil((x)-0.5))

#else
                                      /* platforms unknown */
#error What about a native round function ??

#endif
/*
   --- Bessel
*/
#undef MMATHFCT_HAS_BESSEL
                                      /* platforms w/ native Bessel functions */
#if 0 ||\
    defined(sun) ||\
    ( defined(__hpux) && (OSVERSION >= 900) ) ||\
    defined(__linux__) ||\
    defined(_AIX) ||\
    defined(__osf__) ||\
    defined(__APPLE_CC__)
#define MMATHFCT_HAS_BESSEL
                                      /* platforms w/o native Bessel functions */
#elif 0 ||\
    defined(AMIGA) ||\
    defined(__EMX__)
double j0(double);
double j1(double);
double jn(int,double);
double y0(double);
double y1(double);
double yn(int,double);
#else
                                      /* platforms unknown */
#error What about native Bessel function ??
#endif
double js0(double);
double js1(double);
double jsn(int,double);
double ys0(double);
double ys1(double);
double ysn(int,double);

 
double poisson( double, double );

double e1( double );
 
#undef MMATHFCT_HAS_LGAMMAR
                                      /* platforms w/ native lgamma_r() */
#if 0 ||\
    defined(sun) ||\
    ( defined(__hpux) && (OSVERSION >= 1000) ) ||\
    defined(__linux__) ||\
    defined(_AIX)
#define MMATHFCT_HAS_LGAMMAR 
                                      /* platforms w/o native lgamma_r() */
#elif 0 ||\
    ( defined(__hpux) && (OSVERSION <= 900) ) ||\
    defined(AMIGA) ||\
    defined(__EMX__) ||\
    defined(__osf__) ||\
    defined(__APPLE_CC__)
double lgamma_r( double, int *);
#else
                                      /* platforms unknown */
#error What about a native lgamma_r function ??
#endif

double Gamma   ( double );
double gammainc( double, double );


#if defined(__EMX__) || defined(AMIGA)
double erf(double);
#endif 
double erfinv  ( double );

/*
double asind( double );
double acosd( double );
double atand( double );
double  sind( double );
double  cosd( double );
double  tand( double );
*/ 
int isincosd( int16_t, int16_t [2] );
void sincosd( double, double [2] );
double powi( double, int );
double powd( double, double );
 
#endif
