#ifndef TRXGEO_H
#define TRXGEO_H
/** \file
   Geometry definitions.
*/
/*

   Copyright  2000-2015 M.Kraemer, GSI Darmstadt 

   $Log: trxgeo.h,v $
   Revision 1.4  2015/01/05 15:11:11  kraemer
   streamline 64bit

   Revision 1.3  2008/06/20 14:29:36  kraemer
   tilted vol's

   Revision 1.2  2002/11/25 16:27:27  kraemer
   2file, density, cutoff, e/bfld

   Revision 1.1  2001/01/09 14:54:31  kraemer
   *** empty log message ***


*/

#include "trxvol.h"

                                      /*  */
#define MTRXGEOINC 16

                                      /** \struct STRXGEO
                                          \brief Describes geometry.
                                      */ 
struct STRXGEO {
   unsigned int uiFlag;               /**< flag bits */
#define TRXGEO_ISVALID 0x00000001     /* this geometry is validated */
#define TRXGEO_REFCOPY 0x00000002     /* reference copy, don't free() */
   size_t ulSpace;                    /**< space counter */
   int nV, mV;                        /**< current, max # of volumes in geometry */
   struct STRXVOL *psV;               /**< [mV] volumes */
};
                                      /* */

#define TRXGEO_FILEVERSION 20000118
#define TRXGEO_FILETYPE    "trxgeo"

/*

   Functions:
   ----------

   TRXGeoTargBaseLink ( geo, targbase, flag );
                      o link target materials to volumes
                   15 x to test
                      o calls
                        o TRXVolTargBaseLink

   TRXGeoCutOffDiss   ( geo, xsbase, flag ) 
                      o dissipative cutoff per volume
                   16 o to test
                      o calls
                        o TRXVolCutOffDiss
                     
   TRXGeoCopyByList   ( geoto, geofr, vol-list, flag )
                      o Reference copies of selected volumes
                    7 x to test
                      o calls  
                        o TRXGeoInit()
                        o TRXGeoFree()
                        o TRXGeoFindName()
                        o TRXVolCopy()
 
   TRXGeoDelete       ( geo, vol-list, flag )
                      o delete volumes by name
                   14 x to test
                      o calls:
                        o TRXVolFree()
                        o TRXGeoFree()
                        o TRXGeoSpace()
                           
   TRXGeoExportgd     ( geo, prefix, vol-list, flag )
                      o export in gd format
                   13 o to test
                      ? to rework a la XSAction()
                      o proper file name
                      o calls
                        o TRXGeoCopyByList()   
                        o TRXGeoFree()   
                        o TRXGeoInit()   
                        o TRXGeo2Filegd();
                        
   TRXGeoFindName     ( geo, name, flag )
                      o Locate volume in geometry
                    6 x to test
                      ? rename => GeoLocateVol[Str]
                      o calls                           
                                                    
   TRXGeoFree         ( geo, flag )
                      o Release a geometry's resources
                    4 x to test
                      o calls  
                        o TRXGeoInit()
                        o TRXVolFree()
                                                    
   TRXGeoInit         ( geo, flag ) 
                      o Initialize a geometry 
                    3 x to test
                      o calls  
                        o TRXGeoSpace()
                                                    
   TRXGeoInspect      ( geo, flag )
                      o Inspect a geometry
                    1 x to test
                      o calls  
                        o TRXVolInspect();
                        
   TRXGeoList         ( geo, vol-list, flag )
                      ? to rework a la XSAction()
                    9 o to test
                      o calls  
                        o TRXGeoCopyByList()   
                        o TRXGeoFree()   
                        o TRXGeoInit()   
                        
   TRXGeoRead         ( geo, file-list, flag )
                      o read geometry file(s)
                   11 x to test
                      o calls
                        o TRXGeoFree()
                        o TRXGeoInit()
                        o TRXGeoFindName()
                        o TRXGeoValidate()
                        o TRXGeoSpace()
                        o TRXVol*()
                           
   TRXGeoSpace        ( geo, flag )
                      o estimate space requirements
                    2 x to test
                      o calls  
                        o TRXVolSpace()   
                        
   TRXGeoValidate     ( geo, flag )
                      o Validate a geometry
                    5 x to test
                      o calls  
                        o TRXVolValidate()
                        
   TRXGeoWrite        ( geo, prefix, vol-list, flag )
                      o writes the current geometry setup
                   10 o to test
                      ? to rework a la XSAction()
                      o proper filename from prefix, geo-name?
                      o calls  
                        o TRXGeoCopyByList()   
                        o TRXGeoFree()   
                        o TRXGeoInit()   
                        o TRXGeo2File();

   TRXGeo2File        ( geo, fp, flag )
                      o output geometry data, native format
                    8 x to test
                      o calls
                        o TRXVol*()  

   TRXGeo2Filegd      ( geo, fp, flag )
                      o output geometry data, gd format
                   12 o to do
                      o calls  

   Functions to handle score counters
   ----------------------------------
   
   TRXGeoScoreEvaluate   ( geo, vol-list, nevt, flag )
                       4 x to test
                         o resets counters
                         o calls
                           o TRXVolScoreEvaluate() 

   TRXGeoScoreFanoAlloc  ( geo, nevt, flag )
                       3 x to test
                         o allocates/prepares arrays for Fano factor
                         o calls:
                           o TRXVolScoreFanoAlloc() 
                         
   TRXGeoScoreFanoFree   ( geo, flag )
                       2 x to test
                         o frees Fano arrays
                           o TRXVolScoreFanoFree () 
                         
   TRXGeoScoreReset      ( geo, list, flag )
                       1 x to test 
                         o resets counters 
                           o TRXVolScoreReset() 


   TRXGeoError           error handling
         
*/
                                      /* Common status and action flags */
                                      /* masks */
#define TRXGEO_MASKGLOBAL 0xFF000000  /* global (e.g. debug) */
#define TRXGEO_MASKFUNC   0x00FF0000  /* function specific flags */
#define TRXGEO_MASKQUAL   0x0000FFFF  /* for additional qualifiers */

                                      /* flags */
#define TRXGEO_DEBUG      0x80000000  /* general check flag */
#define TRXGEO_CHECK      TRXGEO_DEBUG/* general check flag */
#define TRXGEO_QUIET      0x10000000  /* suppress error messages */
#define TRXGEO_NOMSG      TRXGEO_QUIET

                                      /* function prototypes */
struct SCMDERR  TRXGeoAction    ( struct STRXGEO *, const char *, const char *, const char *, unsigned int, unsigned int );
struct SCMDERR  TRXGeoCutOffDiss( struct STRXGEO *, struct STRXXSBASE *, unsigned int );
struct SCMDERR  TRXGeoTargBaseLink( struct STRXGEO *, struct STRXTARGBASE *, unsigned int );
struct SCMDERR  TRXGeoCopyByList( struct STRXGEO *, struct STRXGEO *, const char *, unsigned int );
struct SCMDERR  TRXGeoDelete    ( struct STRXGEO *, const char *, unsigned int );
struct SCMDERR  TRXGeoExportgd  ( struct STRXGEO *, const char *, const char *, unsigned int );
struct STRXVOL *TRXGeoFindName  ( struct STRXGEO *, const char *, unsigned int );
struct SCMDERR  TRXGeoFree      ( struct STRXGEO *, unsigned int );
struct SCMDERR  TRXGeoInit      ( struct STRXGEO *, unsigned int );
struct SCMDERR  TRXGeoInspect   ( struct STRXGEO *, unsigned int );
struct SCMDERR  TRXGeoList      ( struct STRXGEO *, const char *, unsigned int );
struct SCMDERR  TRXGeoRead      ( struct STRXGEO *, const char *, unsigned int );
struct SCMDERR  TRXGeoSpace     ( struct STRXGEO *, unsigned int );
struct SCMDERR  TRXGeoValidate  ( struct STRXGEO *, unsigned int );
struct SCMDERR  TRXGeoWrite     ( struct STRXGEO *, const char *, const char *, unsigned int );

struct SCMDERR  TRXGeo2File     ( struct STRXGEO *, FILE *, unsigned int );
struct SCMDERR  TRXGeo2Filegd   ( struct STRXGEO *, FILE *, unsigned int );

struct SCMDERR  TRXGeoScoreEvaluate ( struct STRXGEO *, const char *, unsigned int, unsigned int );
struct SCMDERR  TRXGeoScoreFanoAlloc( struct STRXGEO *, unsigned int, unsigned int );
struct SCMDERR  TRXGeoScoreFanoFree ( struct STRXGEO *, unsigned int );
struct SCMDERR  TRXGeoScoreReset    ( struct STRXGEO *, const char *, unsigned int );

struct STRXGEO _TRXGeoTest( unsigned int );
struct STRXGEO _TRXGeoTestVoxelGrid( unsigned int );
struct STRXGEO _TRXGeoTestBox ( unsigned int );
struct STRXGEO _TRXGeoTestSphere( unsigned int );

                                      /* error handling */
int TRXGeoError( struct SCMDERR *, unsigned int );

enum { ERRTRXGEO_SYS=1,
       ERRTRXGEO_BADADDR,
       ERRTRXGEO_BADNVOL,
       ERRTRXGEO_BADVOL,
       ERRTRXGEO_BADITEMLIST,
       ERRTRXGEO_VOLCOPYFAILED, 
       ERRTRXGEO_NOVOLS,
       ERRTRXGEO_VOLMATCHFAILED, 
       ERRTRXGEO_BADFILETYPE,
       ERRTRXGEO_BADFILEVERSION,
       ERRTRXGEO_BADSYNTAX,
       ERRTRXGEO_BADNUMBER,
       ERRTRXGEO_BADSHAPE,
       ERRTRXGEO_VOLNAMENOTUNIQUE,
       ERRTRXGEO_SHAPENOTYET,
       ERRTRXGEO_BADHEIGHT,
       ERRTRXGEO_BADRADIUS,
       ERRTRXGEO_BADANGLE,
       ERRTRXGEO_BADSIZE,
       ERRTRXGEO_BADTHICKNESS,
       ERRTRXGEO_UNKNOWNTAG,
       ERRTRXGEO_BADNEVT,
       ERRTRXGEO_NOSUCHVOL,   
       ERRTRXGEO_BADDENSITY,
       ERRTRXGEO_BADCUTOFFPROJ,
       ERRTRXGEO_TARGLINKFAILED,
       ERRTRXGEO_CUTOFFDISSFAILED,
       ERRTRXGEO_INVALID,
       ERRTRXGEO_XSBASEINVALID,
};

#endif
